<?php

namespace Teknisa\DF\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\DF\Services\ParametrizacaoFiscalCentralizadaService;
use Doctrine\ORM\EntityManager;

class ParametrizacaoFiscalCentralizadaController extends \Teknisa\DF\Controllers\TeknisaCrud
{
    protected $paramCentralizadaService;
    protected $entityManager;

    public function __construct(ParametrizacaoFiscalCentralizadaService $paramCentralizadaService, EntityManager $entityManager)
    {
        $this->paramCentralizadaService = $paramCentralizadaService;
        $this->entityManager = $entityManager;
    }
    
    public function saveRegra(Request\DataSet $request, Response $response) {
        $row = $request->getDataSet()->getRows()[0];
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $dataSet = new DataSet('regraFiscal', array($row));
            $requestSalvar = new Request\DataSet($dataSet, $request->getMethod(),$request->getRoutePath(), $request->getUserId());
            $this->paramCentralizadaService->saveRegra($requestSalvar);
            $impostosGrupo = $this->paramCentralizadaService->getImpostosGrupo($row['CDGRUPIMPOSTO']);
            $impostosGrupoAntigo = $this->paramCentralizadaService->getImpostosGrupoAntigo($row['CDREGRAFISC']);
            $rowsToSave = array();
            foreach($impostosGrupo as &$imposto) {
                if(!$this->existeImposto($imposto["CDIMPOSTO"], $impostosGrupoAntigo)){
                    $imposto['CDREGRAFISC'] = $row['CDREGRAFISC'];
                    $imposto['IDIMPOBRIGREGRA'] = 'S';
                    $imposto['__is_new'] = true;
                    $rowsToSave[] = $imposto;
                }
            }
            $dataSetImpRegra = new DataSet('impregrafiscal', $rowsToSave);
            $requestImpRegra = new Request\DataSet($dataSetImpRegra, $request->getMethod(),$request->getRoutePath(), $request->getUserId());
            
            $this->paramCentralizadaService->insereImpRegra($requestImpRegra);
            
            $connection->commit();
        }
        catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            $connection->rollback();
        }
    }
    
    
    
    public function existeImposto($cdimposto, $grupoImposto){
        foreach($grupoImposto as $imposto){
            if(in_array($cdimposto, $imposto, true)){
                return true;
            }
        }
        return false;
    }

    public function ConsultaRepetida(Request\Row $request, Response $response)
    {
        try {
            $row = $request->getRow();
            if($row['__is_new']){
                $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('ConsultaRepetida',$this->paramCentralizadaService->ConsultaRepetida($row)));
            }
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deletarRegraImposto(Request\Row $request, Response $response)
    {
        try {
            $row = $request->getRow();
            if (isset($row['CDCLASFISC'])){
                $this->paramCentralizadaService->deletarMensagensImpostoNCM($row['CDREGRAFISC'], $row['CDIMPOSTO'], $row['NRSEQDADOSFISC'], $row['CDCLASFISC']);
                $this->paramCentralizadaService->deletarRegraImpostoNCM($row['CDREGRAFISC'], $row['CDIMPOSTO'], $row['NRSEQDADOSFISC'], $row['CDCLASFISC']);
            }
            elseif (isset($row['CDPRODUTO'])){
                $this->paramCentralizadaService->deletarMensagensImpostoProduto($row['CDREGRAFISC'], $row['CDIMPOSTO'], $row['NRSEQDADOSFISC'], $row['CDPRODUTO']);
                $this->paramCentralizadaService->deletarRegraImpostoProduto($row['CDREGRAFISC'], $row['CDIMPOSTO'], $row['NRSEQDADOSFISC'], $row['CDPRODUTO']);
            }
            else{
                $this->paramCentralizadaService->deletarMensagensImpostoPadrao($row['CDREGRAFISC'], $row['CDIMPOSTO'], $row['NRSEQDADOSFISC']);
                $this->paramCentralizadaService->deletarRegraImpostoPadrao($row['CDREGRAFISC'], $row['CDIMPOSTO'], $row['NRSEQDADOSFISC']);
            }
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deletarRegraCfop(Request\Row $request, Response $response)
    {
        try {
            $row = $request->getRow();
            if (isset($row['CDCLASFISC'])){
                $this->paramCentralizadaService->deletarMensagensCfopNCM($row['CDREGRAFISC'], $row['IDCFOPINTEREST'], $row['CDCLASFISC']);
                $this->paramCentralizadaService->deletarRegraCfopNCM($row['CDREGRAFISC'], $row['IDCFOPINTEREST'], $row['CDCLASFISC']);
            }
            elseif (isset($row['CDPRODUTO'])){
                $this->paramCentralizadaService->deletarMensagensCfopProduto($row['CDREGRAFISC'], $row['IDCFOPINTEREST'], $row['CDPRODUTO']);
                $this->paramCentralizadaService->deletarRegraCfopProduto($row['CDREGRAFISC'], $row['IDCFOPINTEREST'], $row['CDPRODUTO']);
            }
            else{
                $this->paramCentralizadaService->deletarMensagensCfopPadrao($row['CDREGRAFISC'], $row['IDCFOPINTEREST']);
                $this->paramCentralizadaService->deletarRegraCfopPadrao($row['CDREGRAFISC'], $row['IDCFOPINTEREST']);
            }
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
     public function excluirRegra(Request\Row $request, Response $response)
    {
        try {
            $row = $request->getRow();
            
              //  $this->paramCentralizadaService->deletarMensagensCfopPadrao($row['CDREGRAFISC'], $row['IDCFOPINTEREST']);
               $this->paramCentralizadaService->excluirRegra($row['CDREGRAFISC']);
            
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function consultarErrosRegra(Request\Row $request, Response $response)
    {
        try {
            $regra = $request->getRow();
            $erros = $this->paramCentralizadaService->consultarErrosRegra($regra['CDREGRAFISC']);
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('consultarErrosRegra', $erros));
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getNewCodeRegra(Request\Row $request, Response $response) {
        try {
            $newCode = array();
            $newCode = $this->paramCentralizadaService->getNewCodeRegra();
            if($newCode[0]['CODIGO'] == null) $newCode = array("CODIGO" => '0000000001');
            else {
                $newCode = array("CODIGO" => str_pad($newCode[0]['CODIGO'], 10, "0", STR_PAD_LEFT));   
            }
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('getNewCodeRegra', $newCode));
        }
        catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function saveAgrupImposto(Request\Row $request, Response $response) {
        try {
            $regra = $request->getRow();
            $this->paramCentralizadaService->saveAgrupImposto($regra["CDREGRAFISC"], $regra["CDGRUPIMPOSTO"]);
        }
        catch(\Exception $e) {
            
        }
    }
    
    public function validaProdutoRepitidoRegra(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $paisDest = isset($row["CDPAISDEST"]) ? $row["CDPAISDEST"] : '';
            $paisOrig = isset($row["CDPAISORIG"]) ? $row["CDPAISORIG"] : '';
            $sgestadest = isset($row["SGESTADEST"]) ? $row["SGESTADEST"] : '';
            $sgestaori = isset($row["SGESTAORIG"]) ? $row["SGESTAORIG"] : '';
            
            
            $produtos = $this->paramCentralizadaService->validaProdutoRepitidoRegra($row["CDREGRAFISC"], $paisDest, $sgestadest, $paisOrig, $sgestaori, $row["IDTIPODESTFISC"], $row["IDTIPOORIGFISC"], $row["IDUTUFDEST"], $row["IDUTUFORIG"], $row["CDPRODUTO"],  $row["CDIMPOSTO"]);
 
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('validaProdutoRepitidoRegra', array("repetido"=> !empty($produtos))));
        
        }
        catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    } 
    
    public function getMaxdadosfiscprod(Request\Row $request, Response $response) {
        try {            
            $row = $request->getRow();
            $maxDadosFiscProd = $this->paramCentralizadaService->getMaxdadosfiscprod($row["CDREGRAFISC"], $row["CDIMPOSTO"]);
            $maxDadosFiscProd = str_pad($maxDadosFiscProd, 10, "0", STR_PAD_LEFT);
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('getMaxdadosfiscprod', array("NRSEQDADOSFISC" => $maxDadosFiscProd)));
        
        }
        catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getMaxdadosfiscncm(Request\Row $request, Response $response) {
        try {            
            $row = $request->getRow();
            $maxDadosFiscProd = $this->paramCentralizadaService->getMaxdadosfiscncm($row["CDREGRAFISC"], $row["CDIMPOSTO"]);
            $maxDadosFiscProd = str_pad($maxDadosFiscProd, 10, "0", STR_PAD_LEFT);
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('getMaxdadosfiscncm', array("NRSEQDADOSFISC" => $maxDadosFiscProd)));
        
        }
        catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function saveReplicacaoRegraFisc(Request\Row $request, Response $response) {
        
        try {            
            $row = $request->getRow();
            
            $this->paramCentralizadaService->saveReplicacaoRegraFisc($row["CDREGRAFISC"], $row["CDREGRAFISCORIGEM"], $row["DTVALREGRAFISC"]);
            
            if($row["DTVALVINCFISC"] != null && $row["EXPORTARVINCULO"] == "S"){
                $this->paramCentralizadaService->saveVinculoReplicacaoRegraFisc($row["CDREGRAFISC"], $row["CDREGRAFISCORIGEM"], $row["DTVALVINCFISC"]);
            }
        }
        catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
        
        
    }
}

